@@ -4,19 +4,24 @@ Tests general behaviour of the SentenceTransformer class
 
 from __future__ import annotations
 
+import copy
 import json
 import logging
 import os
 import re
+from contextlib import nullcontext
 from functools import partial
 from pathlib import Path
-from typing import Literal, cast
+from typing import Callable, Literal, cast
+from unittest.mock import patch
 
 import numpy as np
 import pytest
 import torch
 from huggingface_hub import CommitInfo, HfApi, RepoUrl
+from tokenizers.processors import TemplateProcessing
 from torch import nn
+from transformers import BertModel
 from transformers.utils import is_peft_available
 
 from sentence_transformers import SentenceTransformer, util
@@ -28,11 +33,12 @@ from sentence_transformers.models import (
     LayerNorm,
     Normalize,
     Pooling,
+    Router,
     Transformer,
     WeightedLayerPooling,
 )
 from sentence_transformers.similarity_functions import SimilarityFunction
-from tests.utils import SafeTemporaryDirectory
+from tests.utils import SafeTemporaryDirectory, is_ci
 
 
 def test_load_with_safetensors() -> None:
@@ -254,9 +260,9 @@ def test_push_to_hub(monkeypatch: pytest.MonkeyPatch, caplog: pytest.LogCaptureF
 
 
 @pytest.mark.parametrize("safe_serialization", [True, False, None])
-def test_safe_serialization(safe_serialization: bool) -> None:
+def test_safe_serialization(stsb_bert_tiny_model: SentenceTransformer, safe_serialization: bool) -> None:
+    model = stsb_bert_tiny_model
     with SafeTemporaryDirectory() as cache_folder:
-        model = SentenceTransformer("sentence-transformers-testing/stsb-bert-tiny-safetensors")
         if safe_serialization:
             model.save(cache_folder, safe_serialization=safe_serialization)
             model_files = list(Path(cache_folder).glob("**/model.safetensors"))
@@ -304,7 +310,7 @@ def test_load_local_without_normalize_directory(stsb_bert_tiny_model: SentenceTr
 
 def test_prompts(stsb_bert_tiny_model: SentenceTransformer, caplog: pytest.LogCaptureFixture) -> None:
     model = stsb_bert_tiny_model
-    assert model.prompts == {}
+    assert model.prompts == {"query": "", "document": ""}
     assert model.default_prompt_name is None
     texts = ["How to bake a chocolate cake", "Symptoms of the flu"]
     no_prompt_embedding = model.encode(texts)
@@ -313,11 +319,11 @@ def test_prompts(stsb_bert_tiny_model: SentenceTransformer, caplog: pytest.LogCa
 
     for query in ["query: ", "query:", "query:   "]:
         # Test prompt="... {}"
-        model.prompts = {}
+        model.prompts = {"query": "", "document": ""}
         assert np.array_equal(model.encode(texts, prompt=query), prompt_embedding)
 
         # Test prompt_name="..."
-        model.prompts = {"query": query}
+        model.prompts = {"query": query, "document": ""}
         assert np.array_equal(model.encode(texts, prompt_name="query"), prompt_embedding)
 
         caplog.clear()
@@ -334,7 +340,7 @@ def test_prompts(stsb_bert_tiny_model: SentenceTransformer, caplog: pytest.LogCa
         with pytest.raises(
             ValueError,
             match=re.escape(
-                "Prompt name 'invalid_prompt_name' not found in the configured prompts dictionary with keys ['query']."
+                "Prompt name 'invalid_prompt_name' not found in the configured prompts dictionary with keys ['query', 'document']."
             ),
         ):
             model.encode(texts, prompt_name="invalid_prompt_name")
@@ -344,7 +350,7 @@ def test_save_load_prompts() -> None:
     with pytest.raises(
         ValueError,
         match=re.escape(
-            "Default prompt name 'invalid_prompt_name' not found in the configured prompts dictionary with keys ['query']."
+            "Default prompt name 'invalid_prompt_name' not found in the configured prompts dictionary with keys ['query', 'document']."
         ),
     ):
         model = SentenceTransformer(
@@ -358,7 +364,7 @@ def test_save_load_prompts() -> None:
         prompts={"query": "query: "},
         default_prompt_name="query",
     )
-    assert model.prompts == {"query": "query: "}
+    assert model.prompts == {"query": "query: ", "document": ""}
     assert model.default_prompt_name == "query"
 
     with SafeTemporaryDirectory() as tmp_folder:
@@ -368,16 +374,16 @@ def test_save_load_prompts() -> None:
         assert config_path.exists()
         with open(config_path, encoding="utf8") as f:
             saved_config = json.load(f)
-        assert saved_config["prompts"] == {"query": "query: "}
+        assert saved_config["prompts"] == {"query": "query: ", "document": ""}
         assert saved_config["default_prompt_name"] == "query"
 
         fresh_model = SentenceTransformer(str(model_path))
-        assert fresh_model.prompts == {"query": "query: "}
+        assert fresh_model.prompts == {"query": "query: ", "document": ""}
         assert fresh_model.default_prompt_name == "query"
 
 
-def test_prompt_output_value_None(stsb_bert_tiny_model_reused) -> None:
-    model = stsb_bert_tiny_model_reused
+def test_prompt_output_value_None(stsb_bert_tiny_model) -> None:
+    model = stsb_bert_tiny_model
     outputs = model.encode(
         ["Text one", "Text two"],
         prompt="query: ",
@@ -397,9 +403,73 @@ def test_prompt_output_value_None(stsb_bert_tiny_model_reused) -> None:
     assert set(outputs[1].keys()) == expected_keys
 
 
+@pytest.mark.parametrize("has_bos_token", [True, False])
+@pytest.mark.parametrize("has_eos_token", [True, False])
+def test_prompt_length_calculation(
+    stsb_bert_tiny_model: SentenceTransformer,
+    has_bos_token: bool,
+    has_eos_token: bool,
+) -> None:
+    # This test checks that the prompt length is calculated correctly, regardless of whether the tokenizer has a
+    # beginning-of-sequence or end-of-sequence token.
+    model = stsb_bert_tiny_model
+    model.prompts = {"query": "Prompt: ", "document": ""}
+    # We need to set this to False, otherwise the prompt length wont be needed:
+    model.set_pooling_include_prompt(False)
+    dummy_bos_token_id = 400
+    dummy_eos_token_id = 500
+    model.tokenizer.cls_token_id = dummy_bos_token_id if has_bos_token else None
+    model.tokenizer.sep_token_id = dummy_eos_token_id if has_eos_token else None
+    if has_bos_token:
+        if has_eos_token:
+            model.tokenizer._tokenizer.post_processor = TemplateProcessing(
+                single="[CLS] $0 [SEP]",
+                special_tokens=[
+                    ("[CLS]", dummy_bos_token_id),
+                    ("[SEP]", dummy_eos_token_id),
+                ],
+            )
+        else:
+            model.tokenizer._tokenizer.post_processor = TemplateProcessing(
+                single="[CLS] $0",
+                special_tokens=[("[CLS]", dummy_bos_token_id)],
+            )
+    else:
+        if has_eos_token:
+            model.tokenizer._tokenizer.post_processor = TemplateProcessing(
+                single="$0 [SEP]",
+                special_tokens=[("[SEP]", dummy_eos_token_id)],
+            )
+        else:
+            model.tokenizer._tokenizer.post_processor = TemplateProcessing(
+                single="$0",
+                special_tokens=[],
+            )
+
+    # Check that we can update the tokenizer in this way
+    if has_eos_token:
+        assert model.tokenize(["dummy text"])["input_ids"].flatten()[-1] == dummy_eos_token_id
+    else:
+        assert model.tokenize(["dummy text"])["input_ids"].flatten()[-1] != dummy_eos_token_id
+
+    if has_bos_token:
+        assert model.tokenize(["dummy text"])["input_ids"].flatten()[0] == dummy_bos_token_id
+    else:
+        assert model.tokenize(["dummy text"])["input_ids"].flatten()[0] != dummy_bos_token_id
+
+    # This should populate _prompt_length_mapping
+    model.encode_query("This is a test sentence")
+
+    only_prompt_length = len(model.tokenizer(["Prompt: "], add_special_tokens=False)["input_ids"][0])
+    if has_bos_token:
+        only_prompt_length += 1
+
+    assert model._prompt_length_mapping == {("Prompt: ", "query"): only_prompt_length}
+
+
 @pytest.mark.skipif(not torch.cuda.is_available(), reason="CUDA must be available to test float16 support.")
-def test_load_with_torch_dtype() -> None:
-    model = SentenceTransformer("sentence-transformers-testing/stsb-bert-tiny-safetensors")
+def test_load_with_torch_dtype(stsb_bert_tiny_model: SentenceTransformer) -> None:
+    model = stsb_bert_tiny_model
 
     assert model.encode(["Hello there!"], convert_to_tensor=True).dtype == torch.float32
 
@@ -440,8 +510,9 @@ def test_load_with_model_kwargs(monkeypatch: pytest.MonkeyPatch) -> None:
 
 
 @pytest.mark.skipif(not is_peft_available(), reason="PEFT must be available to test PEFT support.")
-def test_load_checkpoint_with_peft_and_lora() -> None:
-    from peft import LoraConfig, PeftModel, TaskType
+def test_load_checkpoint_with_peft_and_lora(stsb_bert_tiny_model: SentenceTransformer) -> None:
+    model = stsb_bert_tiny_model
+    from peft import LoraConfig, TaskType
 
     peft_config = LoraConfig(
         target_modules=["query", "key", "value"],
@@ -453,7 +524,6 @@ def test_load_checkpoint_with_peft_and_lora() -> None:
     )
 
     with SafeTemporaryDirectory() as tmp_folder:
-        model = SentenceTransformer("sentence-transformers-testing/stsb-bert-tiny-safetensors")
         model.add_adapter(peft_config)
         model.save(tmp_folder)
         expecteds = model.encode(["Hello there!", "How are you?"], convert_to_tensor=True)
@@ -461,14 +531,16 @@ def test_load_checkpoint_with_peft_and_lora() -> None:
         loaded_peft_model = SentenceTransformer(tmp_folder)
         actuals = loaded_peft_model.encode(["Hello there!", "How are you?"], convert_to_tensor=True)
 
-        assert isinstance(model._modules["0"].auto_model, nn.Module)
-        assert isinstance(loaded_peft_model._modules["0"].auto_model, PeftModel)
+        assert isinstance(model[0].auto_model, nn.Module)
+        assert isinstance(loaded_peft_model[0].auto_model, BertModel)
+        assert loaded_peft_model[0].auto_model._hf_peft_config_loaded
+        assert loaded_peft_model[0].auto_model.active_adapters() == ["default"]
         assert torch.equal(expecteds, actuals)
 
 
 @pytest.mark.skipif(not torch.cuda.is_available(), reason="CUDA must be available to test float16 support.")
-def test_encode_fp16() -> None:
-    tiny_model = SentenceTransformer("sentence-transformers-testing/stsb-bert-tiny-safetensors")
+def test_encode_fp16(stsb_bert_tiny_model: SentenceTransformer) -> None:
+    tiny_model = stsb_bert_tiny_model
     tiny_model.half()
     embeddings = tiny_model.encode(["Hello there!"], convert_to_tensor=True)
     assert embeddings.dtype == torch.float16
@@ -477,40 +549,45 @@ def test_encode_fp16() -> None:
 @pytest.mark.parametrize("convert_to_tensor", [True, False])
 @pytest.mark.parametrize("convert_to_numpy", [True, False])
 @pytest.mark.parametrize(
-    ("precision", "expected_torch_dtype", "expected_numpy_dtype"),
+    ("precision", "expected_torch_dtype", "expected_numpy_dtype", "raises"),
     [
-        (None, torch.float32, np.float32),
-        ("float32", torch.float32, np.float32),
-        ("int8", torch.int8, np.int8),
-        ("uint8", torch.uint8, np.uint8),
-        ("binary", torch.int8, np.int8),
-        ("ubinary", torch.uint8, np.uint8),
+        (None, torch.float32, np.float32, False),
+        ("float32", torch.float32, np.float32, False),
+        ("int8", torch.int8, np.int8, False),
+        ("uint8", torch.uint8, np.uint8, False),
+        ("binary", torch.int8, np.int8, False),
+        ("ubinary", torch.uint8, np.uint8, False),
+        ("error", None, None, True),
     ],
 )
 def test_encode_quantization(
-    stsb_bert_tiny_model_reused: SentenceTransformer,
+    stsb_bert_tiny_model: SentenceTransformer,
     convert_to_tensor: bool,
     convert_to_numpy: bool,
     precision: str,
     expected_torch_dtype,
     expected_numpy_dtype,
+    raises: bool,
 ) -> None:
-    tiny_model = stsb_bert_tiny_model_reused
-    embeddings = tiny_model.encode(
-        ["One sentence", "Another sentence"],
-        convert_to_tensor=convert_to_tensor,
-        convert_to_numpy=convert_to_numpy,
-        precision=precision,
-    )
-    if convert_to_tensor:
-        assert embeddings[0].dtype == expected_torch_dtype
-        assert isinstance(embeddings, torch.Tensor)
-    elif convert_to_numpy:
-        assert embeddings[0].dtype == expected_numpy_dtype
-        assert isinstance(embeddings, np.ndarray)
-    else:
-        assert embeddings[0].dtype == expected_torch_dtype
-        assert isinstance(embeddings, list)
+    tiny_model = stsb_bert_tiny_model
+    with pytest.raises(ValueError, match=f"Precision '{precision}' is not supported") if raises else nullcontext():
+        embeddings = tiny_model.encode(
+            ["One sentence", "Another sentence"],
+            convert_to_tensor=convert_to_tensor,
+            convert_to_numpy=convert_to_numpy,
+            precision=precision,
+        )
+
+    if not raises:
+        if convert_to_tensor:
+            assert embeddings[0].dtype == expected_torch_dtype
+            assert isinstance(embeddings, torch.Tensor)
+        elif convert_to_numpy:
+            assert embeddings[0].dtype == expected_numpy_dtype
+            assert isinstance(embeddings, np.ndarray)
+        else:
+            assert embeddings[0].dtype == expected_torch_dtype
+            assert isinstance(embeddings, list)
 
 
 @pytest.mark.parametrize("sentences", ("Single sentence", ["One sentence", "Another sentence"]))
@@ -519,14 +596,14 @@ def test_encode_quantization(
 @pytest.mark.parametrize("normalize_embeddings", [True, False])
 @pytest.mark.parametrize("output_value", ["sentence_embedding", None])
 def test_encode_truncate(
-    stsb_bert_tiny_model_reused: SentenceTransformer,
+    stsb_bert_tiny_model: SentenceTransformer,
     sentences: str | list[str],
     convert_to_tensor: bool,
     convert_to_numpy: bool,
     normalize_embeddings: bool,
     output_value: Literal["sentence_embedding"] | None,
 ) -> None:
-    model = stsb_bert_tiny_model_reused
+    model = stsb_bert_tiny_model
     embeddings_full_unnormalized: torch.Tensor = model.encode(
         sentences, convert_to_numpy=False, convert_to_tensor=True
     )  # These are raw embeddings which serve as the reference to test against
@@ -607,8 +684,8 @@ def test_encode_truncate(
 
 
 @pytest.mark.parametrize("similarity_fn_name", SimilarityFunction.possible_values())
-def test_similarity_score(stsb_bert_tiny_model_reused: SentenceTransformer, similarity_fn_name: str) -> None:
-    model = stsb_bert_tiny_model_reused
+def test_similarity_score(stsb_bert_tiny_model: SentenceTransformer, similarity_fn_name: str) -> None:
+    model = stsb_bert_tiny_model
     model.similarity_fn_name = similarity_fn_name
     sentences = [
         "The weather is so nice!",
@@ -688,31 +765,40 @@ def test_override_config_versions(stsb_bert_tiny_model: SentenceTransformer) ->
 @pytest.mark.parametrize(
     "modules",
     [
-        [
-            Transformer("sentence-transformers-testing/stsb-bert-tiny-safetensors"),
+        lambda bert_model, word_model: [
+            bert_model[0],
             Pooling(128, "mean"),
             Dense(128, 128),
         ],
-        [Transformer("sentence-transformers-testing/stsb-bert-tiny-safetensors"), CNN(128, 128), Pooling(128, "mean")],
-        [
-            Transformer("sentence-transformers-testing/stsb-bert-tiny-safetensors"),
+        lambda bert_model, word_model: [
+            bert_model[0],
+            CNN(128, 128),
+            Pooling(128, "mean"),
+        ],
+        lambda bert_model, word_model: [
+            bert_model[0],
             Pooling(128, "mean"),
             LayerNorm(128),
         ],
-        [
-            SentenceTransformer("sentence-transformers/average_word_embeddings_levy_dependency")[0],
+        lambda bert_model, word_model: [
+            word_model[0],
             LSTM(300, 128),
             Pooling(128, "mean"),
         ],
-        [
-            Transformer("sentence-transformers-testing/stsb-bert-tiny-safetensors"),
+        lambda bert_model, word_model: [
+            bert_model[0],
             WeightedLayerPooling(128, num_hidden_layers=2, layer_start=1),
             Pooling(128, "mean"),
         ],
-        SentenceTransformer("sentence-transformers/average_word_embeddings_levy_dependency"),
+        lambda bert_model, word_model: word_model,
     ],
 )
-def test_safetensors(modules: list[nn.Module] | SentenceTransformer) -> None:
+def test_safetensors(
+    stsb_bert_tiny_model: SentenceTransformer,
+    avg_word_embeddings_levy: SentenceTransformer,
+    modules: Callable[[], list[nn.Module] | SentenceTransformer],
+) -> None:
+    modules = modules(stsb_bert_tiny_model, avg_word_embeddings_levy)  # Call the lambda to get the actual modules
     if isinstance(modules, SentenceTransformer):
         model = modules
     else:
@@ -734,16 +820,49 @@ def test_safetensors(modules: list[nn.Module] | SentenceTransformer) -> None:
         assert np.allclose(original_embedding, loaded_embedding)
 
 
-def test_empty_encode(stsb_bert_tiny_model: SentenceTransformer) -> None:
+@pytest.mark.parametrize("convert_to_tensor", [True, False])
+@pytest.mark.parametrize("convert_to_numpy", [True, False])
+@pytest.mark.parametrize("normalize_embeddings", [True, False])
+@pytest.mark.parametrize("precision", [None, "float32", "int8", "uint8", "binary", "ubinary"])
+@pytest.mark.parametrize("truncate_dim", [None, 64, 128])
+def test_empty_encode(
+    stsb_bert_tiny_model: SentenceTransformer,
+    convert_to_tensor: bool,
+    convert_to_numpy: bool,
+    normalize_embeddings: bool,
+    precision: str | None,
+    truncate_dim: int | None,
+) -> None:
     model = stsb_bert_tiny_model
-    embeddings = model.encode([])
-    assert embeddings.shape == (0,)
+    embeddings = model.encode(
+        [],
+        convert_to_tensor=convert_to_tensor,
+        convert_to_numpy=convert_to_numpy,
+        normalize_embeddings=normalize_embeddings,
+        precision=precision,
+        truncate_dim=truncate_dim,
+    )
+
+    if convert_to_tensor:
+        assert isinstance(embeddings, torch.Tensor)
+        assert embeddings.numel() == 0
+        assert embeddings.device == model.device
+    elif convert_to_numpy:
+        assert isinstance(embeddings, np.ndarray)
+        assert embeddings.size == 0
+    else:
+        assert embeddings == []
 
 
 @pytest.mark.skipif(not is_peft_available(), reason="PEFT must be available to test adapter methods.")
-def test_multiple_adapters() -> None:
+@pytest.mark.skipif(
+    is_ci(), reason="huggingface_hub & PEFT incorrectly set the user agent in the CI, leading to failures."
+)
+def test_multiple_adapters(
+    stsb_bert_tiny_model: SentenceTransformer, avg_word_embeddings_levy: SentenceTransformer
+) -> None:
     text = "Hello, World!"
-    model = SentenceTransformer("sentence-transformers-testing/stsb-bert-tiny-safetensors")
+    model = stsb_bert_tiny_model
     vec_initial = model.encode(text)
     from peft import LoraConfig, TaskType, get_model_status
 
@@ -800,12 +919,15 @@ def test_multiple_adapters() -> None:
     assert np.allclose(vec_initial, vec_no_adapter)
 
     # Check that for non Transformer-based models we have an error
-    model = SentenceTransformer("sentence-transformers/average_word_embeddings_levy_dependency")
+    model = avg_word_embeddings_levy
     with pytest.raises(ValueError, match="PEFT methods are only supported"):
         model.add_adapter(peft_config)
 
 
 @pytest.mark.skipif(not is_peft_available(), reason="PEFT must be available to test loading PEFT models.")
+@pytest.mark.skipif(
+    is_ci(), reason="huggingface_hub & PEFT incorrectly set the user agent in the CI, leading to failures."
+)
 def test_load_adapter_with_revision():
     model = SentenceTransformer(
         "sentence-transformers-testing/stsb-bert-tiny-lora", revision="3b4f75bcb3dec36a7e05da8c44ee2f7f1d023b1a"
@@ -828,3 +950,313 @@ def test_clip():
     tokenized = model.tokenize(["This is my text sentence"])
     assert "input_ids" in tokenized
     assert tokenized["input_ids"].shape == (1, 5)
+
+
+@pytest.mark.parametrize("sentences", ["Hello world", ["Hello world", "This is a test"], [], [""]])
+@pytest.mark.parametrize("prompt_name", [None, "query", "custom"])
+@pytest.mark.parametrize("prompt", [None, "Custom prompt: "])
+@pytest.mark.parametrize("convert_to_numpy", [True, False])
+@pytest.mark.parametrize("convert_to_tensor", [True, False])
+def test_encode_query(
+    stsb_bert_tiny_model: SentenceTransformer,
+    sentences: str | list[str],
+    prompt_name: str | None,
+    prompt: str | None,
+    convert_to_numpy: bool,
+    convert_to_tensor: bool,
+):
+    model = stsb_bert_tiny_model
+    # Create a mock model with required prompts
+    model.prompts = {"query": "query: ", "custom": "custom: "}
+
+    # Create a mock for the encode method
+    with patch.object(model, "encode", autospec=True) as mock_encode:
+        # Call encode_query
+        model.encode_query(
+            sentences=sentences,
+            prompt_name=prompt_name,
+            prompt=prompt,
+            batch_size=32,
+            convert_to_numpy=convert_to_numpy,
+            convert_to_tensor=convert_to_tensor,
+        )
+
+        # Verify that encode was called with the correct parameters
+        if prompt_name:
+            expected_prompt_name = prompt_name
+        elif prompt is not None:
+            expected_prompt_name = None
+        else:
+            expected_prompt_name = "query"
+
+        mock_encode.assert_called_once()
+        args, kwargs = mock_encode.call_args
+
+        # Check that sentences were passed correctly
+        assert kwargs["sentences"] == sentences
+
+        # Check prompt handling
+        assert kwargs["prompt"] == prompt
+        assert kwargs["prompt_name"] == expected_prompt_name
+
+        # Check other parameters
+        assert kwargs["convert_to_numpy"] == convert_to_numpy
+        assert kwargs["convert_to_tensor"] == convert_to_tensor
+        assert kwargs["task"] == "query"
+
+
+@pytest.mark.parametrize("sentences", ["Hello world", ["Hello world", "This is a test"], [], [""]])
+@pytest.mark.parametrize("prompt_name", [None, "document", "passage", "corpus", "custom"])
+@pytest.mark.parametrize("prompt", [None, "Custom prompt: "])
+@pytest.mark.parametrize("convert_to_numpy", [True, False])
+@pytest.mark.parametrize("convert_to_tensor", [True, False])
+def test_encode_document(
+    stsb_bert_tiny_model: SentenceTransformer,
+    sentences: str | list[str],
+    prompt_name: str | None,
+    prompt: str | None,
+    convert_to_numpy: bool,
+    convert_to_tensor: bool,
+):
+    # Create a mock model with required prompts
+    model = stsb_bert_tiny_model
+    model.prompts = {"document": "document: ", "passage": "passage: ", "corpus": "corpus: ", "custom": "custom: "}
+
+    # Create a mock for the encode method
+    with patch.object(model, "encode", autospec=True) as mock_encode:
+        # Call encode_document
+        model.encode_document(
+            sentences=sentences,
+            prompt_name=prompt_name,
+            prompt=prompt,
+            batch_size=32,
+            convert_to_numpy=convert_to_numpy,
+            convert_to_tensor=convert_to_tensor,
+        )
+
+        # Verify that encode was called with the correct parameters
+        mock_encode.assert_called_once()
+        args, kwargs = mock_encode.call_args
+
+        if prompt_name:
+            expected_prompt_name = prompt_name
+        elif prompt is not None:
+            expected_prompt_name = None
+        else:
+            expected_prompt_name = "document"
+
+        # Check that sentences were passed correctly
+        assert kwargs["sentences"] == sentences
+
+        # Check prompt handling
+        assert kwargs["prompt"] == prompt
+        assert kwargs["prompt_name"] == expected_prompt_name
+
+        # Check other parameters
+        assert kwargs["convert_to_numpy"] == convert_to_numpy
+        assert kwargs["convert_to_tensor"] == convert_to_tensor
+        assert kwargs["task"] == "document"
+
+
+def test_encode_document_prompt_priority(stsb_bert_tiny_model: SentenceTransformer):
+    """Test that proper prompt priority is respected when multiple options are available"""
+    model = stsb_bert_tiny_model
+    model.prompts = {
+        "document": "document: ",
+        "passage": "passage: ",
+        "corpus": "corpus: ",
+    }
+
+    # Create a mock for the encode method
+    with patch.object(model, "encode", autospec=True) as mock_encode:
+        # Call encode_document with no explicit prompt
+        model.encode_document("test")
+
+        # It should select "document" by default since that's first in the priority list
+        args, kwargs = mock_encode.call_args
+        assert kwargs["prompt_name"] == "document"
+
+        # Remove document, should fall back to passage
+        mock_encode.reset_mock()
+        model.prompts = {
+            "passage": "passage: ",
+            "corpus": "corpus: ",
+        }
+        model.encode_document("test")
+        args, kwargs = mock_encode.call_args
+        assert kwargs["prompt_name"] == "passage"
+
+        # Remove passage, should fall back to corpus
+        mock_encode.reset_mock()
+        model.prompts = {
+            "corpus": "corpus: ",
+        }
+        model.encode_document("test")
+        args, kwargs = mock_encode.call_args
+        assert kwargs["prompt_name"] == "corpus"
+
+        # No relevant prompts defined
+        mock_encode.reset_mock()
+        model.prompts = {
+            "query": "query: ",
+        }
+        model.encode_document("test")
+        args, kwargs = mock_encode.call_args
+        assert kwargs["prompt_name"] is None
+
+
+def test_encode_advanced_parameters(stsb_bert_tiny_model: SentenceTransformer):
+    """Test that additional parameters are correctly passed to encode"""
+    model = stsb_bert_tiny_model
+
+    # Create a mock for the encode method
+    with patch.object(model, "encode", autospec=True) as mock_encode:
+        # Call with advanced parameters
+        model.encode_query(
+            "test",
+            normalize_embeddings=True,
+            batch_size=64,
+            show_progress_bar=True,
+            output_value="token_embeddings",
+            precision="uint8",
+            truncate_dim=128,
+            chunk_size=10,
+            custom_param="value",
+        )
+
+        # Verify all parameters were passed correctly
+        args, kwargs = mock_encode.call_args
+        assert kwargs["normalize_embeddings"] is True
+        assert kwargs["batch_size"] == 64
+        assert kwargs["show_progress_bar"] is True
+        assert kwargs["output_value"] == "token_embeddings"
+        assert kwargs["precision"] == "uint8"
+        assert kwargs["truncate_dim"] == 128
+        assert kwargs["chunk_size"] == 10
+        assert kwargs["custom_param"] == "value"
+
+
+@pytest.mark.parametrize("inputs", ["test sentence", ["test sentence"]])
+def test_encode_query_document_vs_encode(stsb_bert_tiny_model: SentenceTransformer, inputs: str | list[str]):
+    """Test the actual integration with encode vs encode_query/encode_document"""
+    # This test requires a real model, but we'll use a small one
+    model = stsb_bert_tiny_model
+    model.prompts = {"query": "query: ", "document": "document: "}
+
+    # Get embeddings with encode_query and encode_document
+    query_embeddings = model.encode_query(inputs)
+    document_embeddings = model.encode_document(inputs)
+
+    # And the same but with encode via prompts (task doesn't help here)
+    encode_query_embeddings = model.encode(inputs, prompt_name="query")
+    encode_document_embeddings = model.encode(inputs, prompt_name="document")
+
+    # With prompts they should be the same
+    np.testing.assert_allclose(query_embeddings, encode_query_embeddings)
+    np.testing.assert_allclose(document_embeddings, encode_document_embeddings)
+
+    # Without prompts they should be different
+    query_embeddings_without_prompt = model.encode(inputs)
+    document_embeddings_without_prompt = model.encode(inputs)
+
+    # Embeddings should differ when different prompts are used
+    with pytest.raises(AssertionError):
+        np.testing.assert_allclose(query_embeddings_without_prompt, query_embeddings)
+    with pytest.raises(AssertionError):
+        np.testing.assert_allclose(document_embeddings_without_prompt, document_embeddings)
+
+
+def test_encode_with_dataset_column(stsb_bert_tiny_model: SentenceTransformer) -> None:
+    """Test that encode can handle a dataset column as input."""
+    model = stsb_bert_tiny_model
+    from datasets import Dataset
+
+    # Create a simple dataset with a text column
+    dataset = Dataset.from_dict({"text": ["This is a test.", "Another sentence."]})
+
+    # Encode the dataset column
+    embeddings = model.encode(dataset["text"], convert_to_tensor=True)
+
+    # Check the shape of the embeddings
+    assert embeddings.shape == (2, model.get_sentence_embedding_dimension())
+
+
+def test_get_model_kwargs(stsb_bert_tiny_model: SentenceTransformer) -> None:
+    """Test that get_model_kwargs returns the correct keyword arguments."""
+    model = stsb_bert_tiny_model
+
+    # Check that the forward kwargs are as expected, i.e. no extra forward kwargs
+    # for this basic model
+    forward_kwargs = model.get_model_kwargs()
+    assert forward_kwargs == []
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "SentenceTransformer.encode() has been called with additional keyword arguments that this model does "
+            "not use: ['normalize']. As per SentenceTransformer.get_model_kwargs(), this model does not accept "
+            "any additional keyword arguments."
+        ),
+    ):
+        # There is no "normalize" argument, this should crash
+        model.encode("Test sentence", normalize=True)
+    # This should run fine
+    model.encode("Test sentence")
+    model.encode_query("Test sentence")
+
+    # If one of the modules has additional forward kwargs, they should be included
+    model[0].forward_kwargs = {"foo"}
+    model[1].forward_kwargs = {"bar", "baz"}
+    assert set(model.get_model_kwargs()) == {"foo", "bar", "baz"}
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "SentenceTransformer.encode() has been called with additional keyword arguments that this model does "
+            "not use: ['normalize']. As per SentenceTransformer.get_model_kwargs(), the valid additional keyword"
+            " arguments are: "
+        )
+        + r"\[('foo'|'bar'|'baz'|, ){5}\].",
+    ):
+        # There is no "normalize" argument, this should crash
+        model.encode("Test sentence", normalize=True)
+    # This should run fine
+    model.encode("Test sentence")
+    model.encode_query("Test sentence")
+    with pytest.raises(
+        TypeError,
+        match=r".*?forward\(\) got an unexpected keyword argument '(foo|bar)'",
+    ):
+        # This would run fine, except the model can't actually accept these arguments (we monkeypatched the modules'
+        # forward_kwargs for this test, after all). The model does send the args down to the underlying modules, though!
+        model.encode("Test sentence", foo=True, bar=False)
+
+    # And also if we have a Router in place
+    query_pooling_copy = copy.deepcopy(model[1])
+    query_pooling_copy.forward_kwargs = {"query_arg"}
+    document_pooling_copy = copy.deepcopy(model[1])
+    document_pooling_copy.forward_kwargs = {"document_arg_1", "document_arg_2"}
+    model[1] = Router.for_query_document(
+        query_modules=[query_pooling_copy],
+        document_modules=[document_pooling_copy],
+    )
+    assert set(model.get_model_kwargs()) == {"foo", "task", "query_arg", "document_arg_1", "document_arg_2"}
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "SentenceTransformer.encode() has been called with additional keyword arguments that this model does "
+            "not use: ['normalize']. As per SentenceTransformer.get_model_kwargs(), the valid additional keyword"
+            " arguments are: "
+        )
+        + r"\[('foo'|'task'|'query_arg'|'document_arg_1'|'document_arg_2'|, ){9}\].",
+    ):
+        # There is no "normalize" argument, this should crash
+        model.encode("Test sentence", task="query", normalize=True)
+    # This should run fine
+    model.encode("Test sentence", task="document")
+    model.encode_query("Test sentence")
+    with pytest.raises(
+        TypeError,
+        match=r".*?forward\(\) got an unexpected keyword argument '(foo|document_arg_1)'",
+    ):
+        # This would run fine, except the model can't actually accept these arguments (we monkeypatched the modules'
+        # forward_kwargs for this test, after all). The model does send the args down to the underlying modules, though!
+        model.encode("Test sentence", task="document", foo=True, document_arg_1=12)
